from typing import Dict, Optional, Union


def example_func(a: Optional[str], b: Union[str, int, None] = None) -> Dict[str, int]:
    """Example function

    Hover over the parameter and return type annotations to see the long versions.

    Args:
        a: An example parameter
        b: Another, with a default
    Returns:
        An example return value
    """
    pass
