@import "theme.css";

/* ReadTheDocs theme global changes */

:root { --accent-color: #f07e44 }
.wy-nav-top { background-color: var(--accent-color) }
.wy-nav-content { max-width: 1200px }
.wy-side-nav-search { background-color: transparent }
.wy-side-nav-search input[type="text"] { border-width: 0 }
.wy-side-nav-search a.icon-home { font-size: 2em; color: var(--accent-color) }
.wy-side-nav-search a.icon-home::before { content: none }
.wy-menu li a { overflow: hidden; text-overflow: ellipsis; }  /* display ellipsis in long items */

/* DocSearch adaptations */

.wy-nav-side { overflow: visible; }
.wy-side-scroll { overflow: inherit; }

/* Making it a bit wider */
.algolia-autocomplete .ds-dropdown-menu {
  max-width: 1000px;
  min-width: 80vw;
}

/* Custom classes */

.small { font-size:40% }
.smaller, .pr, .issue { font-size:70% }


/* Custom classes with bootstrap buttons */

.tutorial,
.tutorial:visited,
.tutorial:hover
{
    /* text-decoration: underline; */
    font-weight: bold;
    padding: 2px 5px;
    white-space: nowrap;
    max-width: 100%;
    background: #EF3270;  /* color from the tail of the scampy */
    border: solid 1px #EF3270;
    border-radius: .25rem;
    font-size: 75%;
    /* font-family: SFMono-Regular,Menlo,Monaco,Consolas,"Liberation Mono","Courier New",Courier,monospace; */
    color: #404040;
    overflow-x: auto;
    box-sizing: border-box;
}


/* Formatting of RTD markup: rubrics and sidebars and admonitions */

/* sidebar */
.rst-content .sidebar {
    /* margin: 0px 0px 0px 12px; */
    padding-bottom: 0px;
}
.rst-content .sidebar p {
    margin-bottom: 12px;
}
.rst-content .sidebar p,
.rst-content .sidebar ul,
.rst-content .sidebar dl {
    font-size: 13px;
}

/* less space after bullet lists in admonitions like warnings and notes */
.rst-content .section .admonition ul {
    margin-bottom: 6px;
}


/* Code: literals and links */

.rst-content tt.literal,
.rst-content code.literal {
    color: #404040;
}
/* slim font weight for non-link code */
.rst-content tt:not(.xref),
.rst-content code:not(.xref),
.rst-content *:not(a) > tt.xref,
.rst-content *:not(a) > code.xref,
.rst-content dl:not(.docutils) code
{
    font-weight: normal;
}
.rst-content a > tt.xref,
.rst-content a > code.xref,
.rst-content dl:not(.docutils) a > tt.xref,
.rst-content dl:not(.docutils) a > code.xref
{
    font-weight: bold;  /* underline looks clumsy, in particular with buttons and
                           other hyperlinks, which don't come with underlines */
}


/* Just one box for annotation code for a less noisy look */

.rst-content .annotation {
    padding: 2px 5px;
    background-color: white;
    border: 1px solid #e1e4e5;
}
.rst-content .annotation tt,
.rst-content .annotation code {
    padding: 0 0;
    background-color: transparent;
    border: 0 solid transparent;
}


/* Parameter lists */

/* Mimick rubric style used for other headings */
/* TODO: once scanpydoc adds classes, also change return types like this */
.rst-content dl:not(.docutils) dl > dt {
    font-weight: bold;
    background: none transparent;
    border-left: none;
    margin: 0 0 12px;
    padding: 3px 0 0;
    font-size: 111.11%;
}
/* Parameters contain <strong> parts and don’t need bold font */
.rst-content dl.field-list dl > dt { font-weight: unset }
/* Add colon between return tuple element name and type */
.rst-content dl:not(.docutils) dl > dt .classifier::before { content: ' : ' }

/* Function headers */

.rst-content dl:not(.docutils) dt {
    background: #edf0f2;
    color: #404040;
    border-top: solid 3px #343131;
}
